<?php

/**
 * @file
 * Admin UI for Wayback module.
 */

function wayback_admin_settings() {
  $form = array();

  $interval = array(0, 60, 300, 0.25*3600, 0.5*3600, 1*3600, 3*3600, 6*3600, 12*3600, 24*3600, 2*24*3600, 3*24*3600, 7*24*3600, 14*24*3600, 28*24*3600);

  $form['wayback_cron_interval'] = array(
    '#type' => 'select',
    '#options' => drupal_map_assoc($interval, 'format_interval'),
    '#title' => t('Cron interval'),
    '#description' => t("This sets the frequency of checking. An interval of 0 means never."),
    '#default_value' => variable_get('wayback_cron_interval', 86400),
  );

  $form['wayback_cache_external_image'] = array(
    '#type' => 'checkbox',
    '#title' => t('Store external images'),
    '#default_value' => variable_get('wayback_cache_external_image', FALSE),
    '#description' => t('If this is checked, we will store external image links'),
  );

  $form['wayback_directory_path'] = array(
    '#type' => 'textfield',
    '#title' => t('File system path'),
    '#default_value' => wayback_directory_path(),
    '#maxlength' => 255,
    '#description' => t('A file system path where the wayback files should be stored. This directory must exist and be writable by Drupal.'),
    '#after_build' => array('system_check_directory'),
  );

  $form['wayback_path_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Path Settings'),
    '#collapsible' => TRUE,
  );

  $form['wayback_path_settings']['wayback_path'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Path'),
    '#default_value' => variable_get('wayback_path', 'wayback'),
    '#description'   => t('[token] may not be used for the first segment of a path.'),
  );

  $form['wayback_path_settings']['wayback_path_tokens'] = array(
    '#value' => theme('token_help', 'wayback'),
  );

  $form = system_settings_form($form);

  $form['#validate'][] = 'wayback_admin_settings_validate';
  $form['#submit'][]   = 'wayback_admin_settings_submit';

  return $form;
}

function wayback_admin_settings_validate($form, &$form_state) {
  $path = $form_state['values']['wayback_path'];
  $first_segment = arg(0, $path);
  if (preg_match('/\[[^\s\]]*\]/', $first_segment)) {
    form_set_error('wayback_path', $first_segment . ' should not be used for the first segment of a path');
  }
}

function wayback_admin_settings_submit($form, &$form_state) {
  menu_rebuild();
}

/**
 * Browse snapshots and manage them.
 */
function wayback_browser(&$form_state) {
  $form = array();
  $result = pager_query("SELECT sid, path, created, status FROM {wayback_page} ORDER BY created desc", 50);

  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $options = array('none' => t('None'), 'delete' => t('Delete'), 'disable' => t('Disable'), 'enable' => t('Enable'));
  $form['options']['operation'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => 'none',
  );
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
    '#submit' => array('wayback_browser_submit'),
  );

  $snapshots = array();
  while ($snapshot = db_fetch_object($result)) {
    $snapshots[$snapshot->sid] = '';
    $form['date'][$snapshot->sid] = array('#value' => format_date($snapshot->created, 'custom', variable_get('date_format_long', 'l, F j, Y - H:i')));
    $form['path'][$snapshot->sid] = array('#value' => l($snapshot->path, url($snapshot->path,  array('absolute' => TRUE))));
    $form['status'][$snapshot->sid] = array('#value' => $snapshot->status);
  }
  $form['snapshots'] = array('#type' => 'checkboxes', '#options' => $snapshots);
  $form['pager'] = array('#value' => theme('pager', NULL, 50, 0));
  $form['#theme'] = 'wayback_browser';
  return $form;
}

function theme_wayback_browser($form) {
  $has_snapshots = isset($form['date']) && is_array($form['date']);
  $select_header = $has_snapshots ? theme('table_select_header_cell') : '';
  $header = array($select_header, t('Date'), t('Path'), t('Status'));

  $output = '';
  $output .= drupal_render($form['options']);
  if ($has_snapshots) {
    foreach (element_children($form['date']) as $key) {
      $row = array();
      $row[] = drupal_render($form['snapshots'][$key]);
      $row[] = drupal_render($form['date'][$key]);
      $row[] = drupal_render($form['path'][$key]);
      $row[] = drupal_render($form['status'][$key]);
      $rows[] = $row;
    }
  }
  else {
    $rows[] = array(array('data' => t('No snapshots available.'), 'colspan' => '3'));
  }

  $output .= theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);
  return $output;
}

/**
 * Validate wayback_browser form submissions.
 *
 * Check if any snapshots have been selected to perform the chosen
 * 'Update option' on.
 */
function wayback_browser_validate($form, &$form_state) {
  if (!$form_state['values']['operation'] || $form_state['values']['operation'] == 'none') {
    form_set_error('', t('No operation selected.'));
  }
  else {
    $snapshots = array_filter($form_state['values']['snapshots']);
    if (count($snapshots) == 0) {
      form_set_error('', t('No items selected.'));
    }
  }
}

/**
 * Process wayback_browser form submissions.
 *
 * Execute the chosen 'Update option' on the selected snapshots.
 */
function wayback_browser_submit($form, &$form_state) {
  switch ($form_state['values']['operation']) {
    case 'delete':
      $snapshots = $form_state['values']['snapshots'];
      $count = 0;
      foreach ($snapshots as $snapshot) {
        if ($snapshot) {
          if (wayback_delete($snapshot)) {
            $count++;
          }
        }
      }
      if ($count) {
        drupal_set_message(format_plural($count, '1 snapshot deleted', '@count snapshots deleted'));
      }
      break;
      case 'enable':
      case 'disable':
        $snapshots = $form_state['values']['snapshots'];
        $count = 0;
        foreach ($snapshots as $snapshot) {
          if ($snapshot) {
            if (wayback_change_status($snapshot, $form_state['values']['operation'])) {
              $count++;
            }
          }
        }
        if ($count) {
          drupal_set_message(format_plural($count, '1 snapshot updated', '@count snapshots updated'));
        }
      break;
  }
}
